local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("SliceCommander")

function SliceCommander:ChangeVolume(val)
	if SC.SET.soundChannel == "Master" then
		SetCVar("Sound_MasterVolume",val)
	elseif SC.SET.soundChannel == "SFX" then
		SetCVar("Sound_SFXVolume",val)
	elseif SC.SET.soundChannel == "Ambience" then
		SetCVar("Sound_AmbienceVolume",val)
	elseif SC.SET.soundChannel == "Music" then
		SetCVar("Sound_MusicVolume",val)
	end
end

function SliceCommander:CreateCore()
	SliceCommander:CreateMoveFrame()
	SliceCommander:CreateEnergyFrame()
	SliceCommander:CreateComboText()
	SliceCommander:CreateSpark()
end

function SliceCommander:MoveStart(frame)
	if SC.SET.IsLocked == false then
		frame:StartMoving()
	end
end

function SliceCommander:MoveStop(frame)
	if SC.SET.IsLocked == false then
		frame:StopMovingOrSizing()
		frame:SetUserPlaced(false)
		SC.OTHER[frame:GetName()]['position'].point,
		SC.OTHER[frame:GetName()]['position'].relativeTo,
		SC.OTHER[frame:GetName()]['position'].relativePoint,
		SC.OTHER[frame:GetName()]['position'].xOfs,
		SC.OTHER[frame:GetName()]['position'].yOfs = frame:GetPoint();
	end
end

function SliceCommander:ApplySkin(isApply)
	if isApply then
		for i,v in pairs(SC.BARS) do
			if i ~= 'CP' then
				if i ~= 'THREAT' and i ~= 'HEALTH' then
					v['obj'].icon:ClearAllPoints()
					v['obj'].icon:SCPoint("RIGHT", v['obj'], "LEFT", -7, 0)
				end
			end
		end
	else
		for i,v in pairs(SC.BARS) do
			if i ~= 'CP' then
				if i ~= 'THREAT' and i ~= 'HEALTH' then
					v['obj'].icon:ClearAllPoints()
					v['obj'].icon:SCPoint("RIGHT", v['obj'], "LEFT", -1, 0)
				end
			end
		end
	end

	SliceCommander:SetTimersHeight(SC.SET.TimerHeight)
	SliceCommander:SetWidth(SC.SET.Width)
	SliceCommander:SetEnergyHeight(SC.SET.EnergyHeight)
	SliceCommander:SetIconSize(SC.SET.IconSize)
end

function SliceCommander:GetVolume()
	if SC.SET.soundChannel == "Master" then
		return GetCVar("Sound_MasterVolume")
	elseif SC.SET.soundChannel == "SFX" then
		return GetCVar("Sound_SFXVolume")
	elseif SC.SET.soundChannel == "Ambience" then
		return GetCVar("Sound_AmbienceVolume")
	elseif SC.SET.soundChannel == "Music" then
		return GetCVar("Sound_MusicVolume")
	end
end

function SliceCommander:OrderTable(tableName)
	local i = 0
	for k, v in pairs(SC_OptionsTable[tableName].args) do
		SC_OptionsTable[tableName].args[k]["order"] = i
		i = i+1
	end
	
	return SC_OptionsTable[tableName]
end

function SliceCommander:InitFont()
	SC.BarFont1 = SliceCommander:NewFont("VTimerFont2", 8, 0.7, 213/255, 200/255, 184/255)
	SC.BarFont2 = SliceCommander:NewFont("VTimerFont4", 8, 1, 213/255, 200/255, 184/255)
	SC.FontIcon = SliceCommander:NewFont("IconFont", 15, 0.5, 1, 1, 1)
end

function SliceCommander:InitIcon()
	SC.OTHER['POISON']['obj'] = SliceCommander:NewIcon("Interface\\Icons\\ability_rogue_dualweild", 'POISON')--Create poison icon warning when it left 5minute or less on the current timer
	SC.OTHER['POISON']['obj']:Show()
	SC.OTHER['POISON']['obj'].button = CreateFrame("Button", nil, UIParent, "SecureActionButtonTemplate")--Create button to reaply poison
	SC.OTHER['POISON']['obj'].button:SetFrameStrata("HIGH")
	SC.OTHER['POISON']['obj'].button:SetFrameLevel(5)
	SC.OTHER['POISON']['obj'].button:SetPoint("BOTTOMLEFT", SC.OTHER['POISON']['obj'], "BOTTOMLEFT")
	SC.OTHER['POISON']['obj'].button:SetPoint("TOPRIGHT", SC.OTHER['POISON']['obj'], "TOPRIGHT")
	SC.OTHER['POISON']['obj'].button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	SC.OTHER['POISON']['obj'].button:SetAttribute("unit", "player")
	SC.OTHER['POISON']['obj'].button:SetAttribute("*type1", "spell")
	SC.OTHER['POISON']['obj'].button:SetAttribute("*type2", "spell")
	SC.OTHER['POISON']['obj'].button:EnableMouse(false)
end

function SliceCommander:InitGCDBar()
	SC.OTHER['AS']['obj'] = SliceCommander:ThinBar("BOTTOMLEFT", "TOPLEFT", 0 , 0)--Create Autoshot
	SC.OTHER['AS']['obj']:SetStatusBarColor(180/255, 38/255, 38/255)
end

function SliceCommander:DisableIcon(obj, val)
	for i,v in pairs(obj.iconBorder) do
		v:Hide()
	end
	obj:SetMinMaxValues(0, val)--Threat value should be between 0 and val
end

function SliceCommander:InitBar()
	SC.BARS['CP']['obj'] = SliceCommander:CPFrame()--create Combo Point bar
	local maxCP = UnitPowerMax("player", Enum.PowerType.ComboPoints)

	for i = 1, maxCP do
		SC.BARS['CP']['obj'].combos[i].bg:SetVertexColor(unpack(SC.SET.CP[maxCP][i]))
	  	SC.BARS['CP']['obj'].combos[i].border[2]:SetBackdropColor(unpack(SC.V.wColor))
	end
	
	for i,v in pairs(SC.BARS) do
		if i ~= "CP" then
			SC.BARS[i].obj = SliceCommander:NewFrame(v.color, i)
		end
	end

	--GLOBAL
	SC.BARS['KSCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_rogue_kidneyshot")
	SC.BARS['TOTCD']['obj'].icon:SetTexture("Interface\\Icons\\Ability_Rogue_TricksOftheTrade")
	SC.BARS['VANCD']['obj'].icon:SetTexture("Interface\\Icons\\Ability_Vanish")
	SC.BARS['MDCD']['obj'].icon:SetTexture("Interface\\Icons\\achievement_bg_killingblow_berserker")
	SC.BARS['DFACD']['obj'].icon:SetTexture("Interface\\Icons\\spell_rogue_deathfromabove")
	SC.BARS['CVCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_rogue_crimsonvial")
	SC.BARS['KICD']['obj'].icon:SetTexture("Interface\\Icons\\ability_kick")
	SC.BARS['COSCD']['obj'].icon:SetTexture("Interface\\Icons\\spell_shadow_nethercloak")
	SC.BARS['EVACD']['obj'].icon:SetTexture("Interface\\Icons\\spell_shadow_shadowward")
	SC.BARS['SPRCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_rogue_sprint")
	SC.BARS['SSCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_rogue_shadowstep")
	SC.BARS['FECD']['obj'].icon:SetTexture("Interface\\Icons\\ability_rogue_feint")
	
	--COVENANTS
	SC.BARS['SUMSCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_kyrian_summonsteward")
	SC.BARS['FCCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_necrolord_fleshcraft")
	SC.BARS['SOSCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_nightfae_flicker")
	SC.BARS['DOSCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_venthyr_doorofshadows")
	SC.BARS['ERCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_bastion_rogue")
	SC.BARS['SBSCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_maldraxxus_rogue")
	SC.BARS['SPCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_ardenweald_rogue")
	SC.BARS['FLACD']['obj'].icon:SetTexture("Interface\\Icons\\ability_revendreth_rogue")


	--ASSASSINATION
	SC.BARS['VENCD']['obj'].icon:SetTexture("Interface\\Icons\\Ability_Rogue_Deadliness")
	SC.BARS['EXCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_deathwing_bloodcorruption_earth")
	SC.BARS['GARCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_rogue_garrote")
	SC.BARS['KBCD']['obj'].icon:SetTexture("Interface\\Icons\\inv_knife_1h_artifactgarona_d_01")

	--OUTLAW
	SC.BARS['ARCD']['obj'].icon:SetTexture("Interface\\Icons\\Spell_Shadow_ShadowWordDominate")
	SC.BARS['KSPCD']['obj'].icon:SetTexture("Interface\\Icons\\Ability_Rogue_Murderspree")
	SC.BARS['GHCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_rogue_grapplinghook")
	SC.BARS['BECD']['obj'].icon:SetTexture("Interface\\Icons\\inv_weapon_rifle_01")
	SC.BARS['RB']['obj'].icon:SetTexture("Interface\\Icons\\ability_rogue_rollthebones")
	SC.BARS['RBCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_rogue_rollthebones")
	SC.BARS['CDBCD']['obj'].icon:SetTexture("Interface\\Icons\\inv_sword_1h_artifactskywall_d_01dual")
	SC.BARS['BFCD']['obj'].icon:SetTexture("Interface\\Icons\\Ability_Warrior_PunishingBlow")
	SC.BARS['GSCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_creature_cursed_02")
	SC.BARS['BRCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_arakkoa_spinning_blade")
	SC.BARS['GGCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_gouge")

	--SUBTLETY
	SC.BARS['SDCD']['obj'].icon:SetTexture("Interface\\Icons\\Ability_Rogue_Shadowdance")
	SC.BARS['SBCD']['obj'].icon:SetTexture("Interface\\Icons\\inv_knife_1h_grimbatolraid_d_03")
	SC.BARS['GBCD']['obj'].icon:SetTexture("Interface\\Icons\\inv_knife_1h_artifactfangs_d_01")
	SC.BARS['SODCD']['obj'].icon:SetTexture("Interface\\Icons\\spell_shadow_rune")
	SC.BARS['SHTCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_rogue_throwingspecialization")
	SC.BARS['STHCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_rogue_sinistercalling")

	SliceCommander:DisableIcon(SC.BARS['THREAT']['obj'], 110)
	SliceCommander:DisableIcon(SC.BARS['HEALTH']['obj'], 100)
	SliceCommander:DisableIcon(SC.BARS['THEALTH']['obj'], 100)
end

function SliceCommander:InitConstantValue()
	local height, width
	height = tonumber(string.match(GetCVar("gxWindowedResolution"), "%d+x(%d+)"))
	width = tonumber(string.match(GetCVar("gxWindowedResolution"), "(%d+)x+%d"))
	width = width*SC.SET.eyefinity--eyefinity is wired
	SC.curCombo = 0
	SC.RUP_AlertPending = 3
	SC.SND_AlertPending = 3
	SC.V = {
		['bColor'] = {.1,.1,.1},
		['wColor'] = {.6,.6,.6},
		['pxH'] = UIParent:GetHeight() / height,
		['pxW'] = UIParent:GetWidth() / width,
	}

	SC.Spark = {}
end

function SliceCommander:ShowHideBorder(visible)
	if visible then
		for i,v in pairs(VTimerEnergy.border) do
			v:Show()
		end
		for i,v in pairs(SC.BARS) do
			if i ~= "CP" then
				for j,w in pairs(v['obj'].border) do
					w:Show()
				end
				if i ~= "THREAT" and i ~= "HEALTH" and i ~= "THEALTH" then
					for j,w in pairs(v['obj'].iconBorder) do
						w:Show()
					end
				end
			end
		end
		for i,v in pairs(SC.BARS['CP']['obj'].combos) do
			for j,w in pairs(v.border) do
				w:Show()
			end
		end
	else
		for i,v in pairs(VTimerEnergy.border) do
			v:Hide()
		end
		for i,v in pairs(SC.BARS) do
			if i ~= "CP" then
				for j,w in pairs(v['obj'].border) do
					w:Hide()
				end
				if i ~= "THREAT" and i ~= "CP" and i ~= "HEALTH" and i ~= "THEALTH" then
					for j,w in pairs(v['obj'].iconBorder) do
						w:Hide()
					end
				end
			end
		end
		for i,v in pairs(SC.BARS['CP']['obj'].combos) do
			for j,w in pairs(v.border) do
				w:Hide()
			end
		end
	end
end

function SliceCommander:ApplyConf()
	SliceCommander:LockCkecked(SC.SET.IsLocked)
	SliceCommander:SetTimersHeight(SC.SET.TimerHeight)
	SliceCommander:SetWidth(SC.SET.Width)
	SliceCommander:SetEnergyHeight(SC.SET.EnergyHeight)
	SliceCommander:SetIconSize(SC.SET.IconSize)
	SliceCommander:Config_RetextureBars(SliceCommander:BarTexture())
	SliceCommander:FrameOpacity(SC.SET.FrameOpacity)
	SliceCommander:SetTransparency(SC.SET.EnergyTrans)
	SliceCommander:ApplySkin(SC.SET.skin)
	SliceCommander:ShowHideBorder(SC.SET.skin)
	SliceCommander:ACTIVE_TALENT_GROUP_CHANGED()
end

function SliceCommander:SetSparks()
	local spec = GetSpecialization()
	
	SC.Spark[1]:SCPoint("TOPLEFT", VTimerEnergy, "TOPLEFT", ((SC.SET.Energy[spec][1] / UnitPowerMax("player", Enum.PowerType.Energy) * SC.SET.Width)-1), 0)
	SC.Spark[2]:SCPoint("TOPLEFT", VTimerEnergy, "TOPLEFT", ((SC.SET.Energy[spec][2] / UnitPowerMax("player", Enum.PowerType.Energy) * SC.SET.Width)-1), 0)
	SC.Spark[3]:SCPoint("TOPLEFT", VTimerEnergy, "TOPLEFT", ((SC.SET.Energy[spec][3] / UnitPowerMax("player", Enum.PowerType.Energy) * SC.SET.Width)-1), 0)
end

function SliceCommander:LockCkecked(checked)
	if checked then
		SliceCommander:SetTransparency(SC.SET.FrameOpacity)
		SliceCommanderFrame:SetAlpha(0)
		SliceCommanderFrame:EnableMouse(false)
		SC.OTHER['POISON']['obj']['moveframe']:SetAlpha(0)
		SC.OTHER['POISON']['obj']['moveframe']:EnableMouse(false)
		SC.SET.IsLocked = true
	else
		SliceCommander:SetTransparency(100)
		SliceCommanderFrame:SetAlpha(.5)
		SliceCommanderFrame:EnableMouse(true)
		SC.OTHER['POISON']['obj']['moveframe']:SetAlpha(.5)
		SC.OTHER['POISON']['obj']['moveframe']:EnableMouse(true)
		SC.OTHER['POISON']['obj']:SetAlpha(1)
		SC.OTHER['POISON']['obj']:Show()
		SC.SET.IsLocked = false
		SliceCommander:StaticPopup_Show('CONFIG_UNLOCK')
	end
end

function SliceCommander:Config_RetextureBars(texture)
	VTimerEnergy:SetStatusBarTexture(texture)
	for i,v in pairs(SC.BARS) do
		if v['expire'] ~= nil and i ~= 'CP' then
			v['obj']:SetStatusBarTexture(texture)
		end
	end

	for i = 1, 10 do
		SC.BARS['CP']['obj'].combos[i].bg:SetTexture(texture)
	end
end

function SliceCommander:SetWidth(w)
	local pxMargin = 1
	local maxCP = UnitPowerMax("player", Enum.PowerType.ComboPoints)
	if SC.SET.skin then
		pxMargin = 7
	end
	local widthCP = math.floor((w-(pxMargin*(maxCP-1)))/maxCP)
	w = widthCP*maxCP+pxMargin*(maxCP-1)

	local h = SC.SET.TimerHeight

	if SC.SET.skin then
		--3 pixel border x2 + 1 pixel margin
		h = h + 6 + 1
	end
	for i,v in pairs(SC.BARS) do
		if v['expire'] ~= nil and i ~= 'THREAT' and i ~= 'CP' and i ~= "HEALTH" and i ~= "THEALTH" then
			SliceCommander:SetSizeBorder(w-h, 0, v['obj'].border)
			v['obj']:SCWidth(w-h)
		end
	end

	VTimerEnergy:SCWidth(w)
	SliceCommander:SetSizeBorder(w, 0, VTimerEnergy.border)
	SliceCommanderFrame:SCWidth(w+16)

	SC.OTHER['AS']['obj']:SCWidth(w)
	SC.BARS['CP']['obj']:SCWidth(w)

	SC.BARS['THREAT']['obj']:SCWidth(w)
	SliceCommander:SetSizeBorder(w, 0, SC.BARS['THREAT']['obj'].border)
	SC.BARS['HEALTH']['obj']:SCWidth(w)
	SliceCommander:SetSizeBorder(w, 0, SC.BARS['HEALTH']['obj'].border)
	SC.BARS['THEALTH']['obj']:SCWidth(w)
	SliceCommander:SetSizeBorder(w, 0, SC.BARS['THEALTH']['obj'].border)

	local cx = 0
	for i = 1, maxCP do
		SC.BARS['CP']['obj'].combos[i]:ClearAllPoints()
		SC.BARS['CP']['obj'].combos[i]:SCPoint("TOPLEFT", SC.BARS['CP']['obj'], "TOPLEFT", cx, 0)
		SC.BARS['CP']['obj'].combos[i]:SCPoint("BOTTOMRIGHT", SC.BARS['CP']['obj'], "BOTTOMLEFT", cx + widthCP , 0)
		SC.BARS['CP']['obj'].combos[i]:SCWidth(widthCP)
		SliceCommander:SetSizeBorder(widthCP, 0, SC.BARS['CP']['obj'].combos[i].border)
		cx = cx + widthCP + pxMargin
	end
	SliceCommander:SetSparks()
end

function SliceCommander:SetSizeBorder(w, h, border)
	local k = 6
	for i,v in pairs(border) do
		if h ~= 0 then
			v:SCHeight(h+k)
		end
		if w ~= 0 then
			v:SCWidth(w+k)
		end
		k = k-2
	end
end

function SliceCommander:SetEnergyHeight(h)
	VTimerEnergy:SCHeight(h)
	SliceCommanderFrame:SCHeight(h+16)
	VTimerEnergyTxt:SCHeight(h)
	SliceCommander_Combo:SCHeight(h)
	SC.Spark[1]:SCHeight(h)
	SC.Spark[2]:SCHeight(h)
	SC.Spark[3]:SCHeight(h)
	SliceCommander:SetSizeBorder(0, h, VTimerEnergy.border)
	SliceCommander:HealthBar('HEALTH')
	SliceCommander:HealthBar('THEALTH')
end

function SliceCommander:SetIconSize(w)
	SliceCommander:SetIconSizeObj(SC.OTHER['POISON']['obj'], w)
end

function SliceCommander:SetIconSizeObj(obj, w)
	obj:SCSize(w, w)
	obj.overlay:SCSize(w, w)
	obj.iconTxt:SCSize(w, w-5)
	obj.icon:SCSize(w, w)
	obj.moveframe:SCSize(w+16, w+16)
	SliceCommander:SetSizeBorder(w, w, obj.border)
end

function SliceCommander:SetTimersHeight(h)
	for i,v in pairs(SC.BARS) do
		if v['expire'] ~= nil and i ~= 'CP' then
			v['obj']:SCHeight(h)
			v['obj'].text:SCHeight(h)
			v['obj'].text2:SCHeight(h)

			SliceCommander:SetSizeBorder(0, h, v['obj'].border)
			if i ~= 'THREAT' and i ~= 'HEALTH' and i ~= 'THEALTH' then
				v['obj'].icon:SCSize(h, h)
				SliceCommander:SetSizeBorder(h, h, v['obj'].iconBorder)
			end
		end
	end

	if _G['SCtick'] == nil then _G['SCtick'] = {} end
	for k, v in pairs(_G['SCtick']) do
		v:SCHeight(h)
	end

	SC.BARS['CP']['obj']:SCHeight(h)
	SC.BARS['CP']['obj'].comboText:SCHeight(h)
	local maxCP = UnitPowerMax("player", Enum.PowerType.ComboPoints)
	for i = 1, maxCP do
		SC.BARS['CP']['obj'].combos[i]:SCHeight(h)
		SliceCommander:SetSizeBorder(0, h, SC.BARS['CP']['obj'].combos[i].border)
	end

	SC.BarFont1:SetFont(SliceCommander:GetFont(), (h-2)*SC.V.pxH)
	SC.BarFont2:SetFont(SliceCommander:GetFont(), (h-2)*SC.V.pxH)
	SC.BARS['CP']['obj'].comboText:SetFont(SliceCommander:GetFont(), (h-2)*SC.V.pxH, "THINOUTLINE")
	SliceCommander:SetWidth(SC.SET.Width)
end

function SliceCommander:GetFont()
	local LSM = LibStub("LibSharedMedia-3.0")
	return LSM:Fetch("font", SC.SET.font)
end

function SliceCommander:FrameOpacity(opValue)
	SliceCommander:SetTransparency(opValue)

	for i,v in pairs(SC.BARS) do
		if v['expire'] ~= nil then
			v['obj']:SetAlpha(opValue/100)
		end
	end
end

function SliceCommander:TestCurrentVersion()
	if _G['SC'].version == nil or _G['SC'].version < '7.00' then
		_G['SC'] = {version = GetAddOnMetadata("SliceCommander", "Version")}
	else
		if _G['SC'].version < '9.01' then
			_G['SC'].SET.Pos['RIP'] = nil--Remove ripost
			_G['SC'].BARS['RIP'] = nil
			_G['SC'].SET.Pos['RIPCD'] = nil
			_G['SC'].BARS['RIPCD'] = nil
			_G['SC'].SET.Pos['VAN'] = nil--ADD CD TO the NAME OF SOME CD
			_G['SC'].BARS['VAN'] = nil
			_G['SC'].SET.Pos['CD'] = nil
			_G['SC'].BARS['CD'] = nil
			_G['SC'].SET.Pos['MD'] = nil
			_G['SC'].BARS['MD'] = nil
			_G['SC'].SET.Pos['TOT'] = nil
			_G['SC'].BARS['TOT'] = nil
			_G['SC'].SET.Pos['KI'] = nil
			_G['SC'].BARS['KI'] = nil
			_G['SC'].SET.Pos['DFA'] = nil
			_G['SC'].BARS['DFA'] = nil
			_G['SC'].SET.Pos['EX'] = nil
			_G['SC'].BARS['EX'] = nil
			_G['SC'].SET.Pos['KSP'] = nil
			_G['SC'].BARS['KSP'] = nil
			_G['SC'].SET.Pos['BR'] = nil
			_G['SC'].BARS['BR'] = nil
			_G['SC'].SET.Pos['GH'] = nil
			_G['SC'].BARS['GH'] = nil
			_G['SC'].SET.Pos['STH'] = nil
			_G['SC'].BARS['STH'] = nil
		end
		if _G['SC'].version < '9.06' then
			_G['SC'].SET.Pos['TXBCD'] = nil
			_G['SC'].BARS['TXBCD'] = nil
			_G['SC'].SET.Pos['TXB'] = nil
			_G['SC'].BARS['TXB'] = nil
			_G['SC'].SET.Pos['NB'] = nil
			_G['SC'].BARS['NB'] = nil
			_G['SC'].SET.Pos['SOTD'] = nil
			_G['SC'].BARS['SOTD'] = nil
			_G['SC'].SET.Pos['FOTD'] = nil
			_G['SC'].BARS['FOTD'] = nil
		end
		
		_G['SC'].version = GetAddOnMetadata("SliceCommander", "Version")
	end
end

function SliceCommander:DisplaySpells(Spells)
	for i, v in ipairs(Spells) do
		if SC.SET.display[v] == nil then
			SC.SET.display[v] = true
		end
	end
end

function SliceCommander:NotDisplaySpells(Spells)
	for i, v in ipairs(Spells) do
		if SC.SET.display[v] == nil then
			SC.SET.display[v] = false
		end
	end
end

function SliceCommander:FlashSpells(Spells)
	for i, v in ipairs(Spells) do
		if SC.SET.flash[v] == nil then
			SC.SET.flash[v] = true
		end
	end
end

function SliceCommander:NotFlashSpells(Spells)
	for i, v in ipairs(Spells) do
		if SC.SET.flash[v] == nil then
			SC.SET.flash[v] = false
		end
	end
end

function SliceCommander:OrderSpells(Spells)
	for i, v in pairs(Spells) do
		_G['SC'].orderBar = _G['SC'].orderBar+1
		if _G['SC'].orderBar > _G['SC'].maxPos then
			_G['SC'].maxPos = _G['SC'].orderBar
		end
		if SC.SET.Pos[v] == nil then
			SC.SET.Pos[v] = _G['SC'].orderBar
		end
	end
end

function SliceCommander:OrderCDs(Spells)
	for i, v in pairs(Spells) do
		_G['SC'].orderCDBar = _G['SC'].orderCDBar-1
		if _G['SC'].orderCDBar < _G['SC'].maxCDPos then
			_G['SC'].maxCDPos = _G['SC'].orderCDBar
		end
		if SC.SET.Pos[v] == nil then
			SC.SET.Pos[v] = _G['SC'].orderCDBar
		end
	end

end

function SliceCommander:Global()
	SliceCommander:DisplaySpells({'CP', 'THREAT', 'SND', 'CS', 'KS', 'KSCD', 'CV', 'CVCD', 'TOTCD', 'FE', 'SSCD', 'SUB', 'CDCD', 'AC', 'MDCD', 'DFACD', 'Poison', 'MAI', 'FECD', 'ESSECD', 'DPICON', 'SUMSCD', 'FCCD', 'SOSCD', 'DOSCD', 'ERCD', 'SBSCD', 'SBS', 'SPCD', 'FLACD', 'SP', 'FLA'})
	SliceCommander:NotDisplaySpells({'HEALTH', 'THEALTH', 'VANCD', 'KICD', 'COS', 'COSCD', 'EVA', 'EVACD', 'SPR', 'SPRCD', 'SS'})
	
	SliceCommander:FlashSpells({'SND'})
	SliceCommander:NotFlashSpells({'KSCD', 'CVCD', 'TOTCD', 'SSCD', 'CDCD', 'MDCD', 'DFACD', 'FECD', 'ESSECD', 'VANCD', 'KICD', 'COSCD', 'EVACD', 'SPRCD', 'SUMSCD', 'FCCD', 'SOSCD', 'DOSCD', 'ERCD', 'SBSCD', 'SPCD', 'FLACD', 'SP', 'FLA'})

	_G['SC'].orderBar = _G['SC'].maxPos
	_G['SC'].orderCDBar = _G['SC'].maxCDPos
	
	if SC.SET.Pos.CP == nil then--Combo Points
		SC.SET.Pos.CP = -1
	end
	
	SliceCommander:OrderSpells({'SND', 'CV', 'FE', 'SUB', 'AC', 'MAI', 'COS', 'EVA', 'SPR', 'SS', 'SP', 'FLA', 'SBS'})
	SliceCommander:OrderCDs({'CVCD', 'TOTCD', 'VANCD', 'SSCD', 'CDCD', 'MDCD', 'DFACD', 'THREAT', 'HEALTH', 'THEALTH', 'CS', 'KS', 'KSCD', 'KICD', 'COSCD', 'EVACD', 'SPRCD', 'FECD', 'ESSECD', 'SUMSCD', 'FCCD', 'SOSCD', 'DOSCD', 'ERCD', 'SBSCD', 'SPCD', 'FLACD'})
end

function SliceCommander:Assassination()
	SliceCommander:DisplaySpells({'RUP', 'GAR', 'GARCD', 'VEN', 'VENCD', 'ENV', 'EP', 'DPBUT', 'EXCD', 'KB', 'KBCD', 'BA', 'ST', 'BLS', 'MA', 'CT'})
	SliceCommander:NotDisplaySpells({'DP'})
	
	SliceCommander:FlashSpells({'VENCD', 'EXCD'})
	SliceCommander:NotFlashSpells({'GARCD', 'EXCD', 'KBCD'})

	_G['SC'].orderBar = 0
	_G['SC'].orderCDBar = -1
	
	SliceCommander:OrderSpells({'GAR', 'RUP', 'ENV', 'KB', 'BA', 'ST', 'VEN', 'DP', 'EP', 'BLS', 'MA', 'CT'})
	SliceCommander:OrderCDs({'VENCD', 'KBCD', 'EXCD', 'GARCD'})
end

function SliceCommander:Outlaw()
	SliceCommander:DisplaySpells({'RB', 'AR', 'ARCD', 'CDB', 'CDBCD', 'KSPCD', 'GS', 'GHCD', 'BE', 'BECD', 'BF', 'BFCD', 'OP', 'GSWW', 'LD', 'GSCD', 'BRCD', 'RBCD'})
	SliceCommander:NotDisplaySpells({'BFICON', 'GG', 'GGCD'})
	
	SliceCommander:FlashSpells({'ARCD', 'CDBCD', 'KSPCD', 'BECD', 'BRCD', 'RBCD'})
	SliceCommander:NotFlashSpells({'GHCD', 'BFCD', 'GSCD'})

	_G['SC'].orderBar = 0
	_G['SC'].orderCDBar = -1
	
	SliceCommander:OrderSpells({'RB', 'AR', 'GS', 'OP', 'CDB', 'GSWW', 'BF', 'LD'})
	SliceCommander:OrderCDs({'BECD', 'BE', 'ARCD', 'BFCD', 'KSPCD', 'GHCD', 'CDBCD', 'GSCD', 'BRCD', 'GG', 'GGCD', 'RBCD'})
end

function SliceCommander:Subtlety()
	SliceCommander:DisplaySpells({'SD', 'SDCD', 'SB', 'SBCD', 'SOD', 'GB', 'GBCD', 'SODCD', 'SHT', 'SHTCD', 'STHCD'})
	SliceCommander:NotDisplaySpells({'FW'})
	
	SliceCommander:FlashSpells({'SDCD', 'SBCD', 'SODCD'})
	SliceCommander:NotFlashSpells({'GBCD', 'SHTCD', 'STHCD'})

	_G['SC'].orderBar = 0
	_G['SC'].orderCDBar = -1
	
	SliceCommander:OrderSpells({'GB', 'SOD', 'SD', 'SB', 'SHT', 'FW'})
	SliceCommander:OrderCDs({'GBCD', 'SDCD', 'SBCD', 'SODCD', 'SHTCD', 'STHCD'})
	
end

function SliceCommander:SetEssence()
	local essenceId = C_AzeriteEssence.GetMilestoneEssence(115)
	if not essenceId then
		return nil
	end
	local essenceRank = C_AzeriteEssence.GetEssenceInfo(essenceId)['rank']
	
	if SC_SPELLID['ALLESSE'][essenceId] ~= nil then
		SC_SPELLID['ESSECD'] = SC_SPELLID['ALLESSE'][essenceId][essenceRank]
		local name, _, icon = GetSpellInfo(SC_SPELLID['ESSECD'])
		SC_SPELL['ESSECD'] = name
		if SC.BARS['ESSECD']['obj'] ~= nil and SC.BARS['ESSECD']['obj'] ~= 0 then
			SC.BARS['ESSECD']['obj'].icon:SetTexture(icon)
		end
	else
		SC_SPELLID['ESSECD'] = -1
		SC_SPELL['ESSECD'] = 'ESSENCE'
	end
	SC.BARS['ESSECD']['name'] = SC_SPELL['ESSECD']
	SC.BARS['ESSECD']['id'] = SC_SPELLID['ESSECD']
end

function SliceCommander:DisplayOrderBar()
	SliceCommander:Assassination()
	SliceCommander:Outlaw()
	SliceCommander:Subtlety()
	SliceCommander:Global()
end

function SliceCommander:SetSetOptions()
	local defaultTable = {'poison'}
	for k, v in pairs(defaultTable) do
		if SC.SET[v] == nil then
			SC.SET[v] = {}
		end
	end
	local defaultEnabled = {'PadLatency', 'SpellText', 'SoundCombat', 'ThreatGroupOnly', 'kickEnable', 'displayGCD'}
	for k, v in pairs(defaultEnabled) do
		if SC.SET[v] == nil then
			SC.SET[v] = true
		end
	end
	local defaultDisabled = {'IsLocked', 'HideEnergy', 'FullTimer', 'ARMarioMod', 'skin'}
	for k, v in pairs(defaultDisabled) do
		if SC.SET[v] == nil then
			SC.SET[v] = false
		end
	end
end

function SliceCommander:SetRB()
	if SC.SET.RB == nil then
		SC.SET.RB = {
			['SC'] = 'SS',
			['GM'] = 'AS',
			['RP'] = 'Crit',
			['TB'] = 'CDR',
			['BT'] = 'NRJ',
			['BS'] = 'CP+'
		}
		SC.SET.display.RBName = true
	end
	SC.BARS['RB'].subbar = {
		[SC_SPELLID['SC']] = 'SC',
		[SC_SPELLID['GM']] = 'GM',
		[SC_SPELLID['RP']] = 'RP',
		[SC_SPELLID['TB']] = 'TB',
		[SC_SPELLID['BT']] = 'BT',
		[SC_SPELLID['BS']] = 'BS'
	}
end

function SliceCommander:TestValueOptions()
	if SC.SET == nil or SC.SET.display == nil or SC.SET.Pos == nil then
		SC.SET = { display = {}, Pos = {} }
	end
	if SC.SET.flash == nil then
		SC.SET.flash = {}
	end
	if _G['SC'].SET.PixelPerfect == nil then
		_G['SC'].SET.PixelPerfect = true
	end
	SliceCommander:DisplayOrderBar()
	SliceCommander:SetSetOptions()
	if SC.SET.Energy == nil then
		SC.SET.Energy = {
			[1] = {--ASSASSINATION
				[1] = 25,
				[2] = 35,
				[3] = 55,
			},
			[2] = {--OUTLAW
				[1] = 25,
				[2] = 35,
				[3] = 50,
			},
			[3] = {--SUBTLETY
				[1] = 25,
				[2] = 35,
				[3] = 40,
			}
		}
	end
	if SC.SET.kickMessage == nil then
		SC.SET.kickMessage = "Kick successful"
	end
	if SC.SET.kickSpell == nil then
		SC.SET.kickSpell = true
	end
	if SC.SET.kickChannel == nil then
		SC.SET.kickChannel = {
			['INSTANCE_CHAT'] = true,
			['RAID'] = true,
			['PARTY'] = true,
			['SAY'] = true
		}
	end
	if SC.SET.eyefinity == nil then
		SC.SET.eyefinity = 1
	end
	if SC.SET.HEALTH == nil then
		SC.SET.HEALTH = {
			value = false,
			HideCombat = true,
			MidUnder = 'None',
			LowUnder = 'None'
		}
	end
	if SC.SET.THEALTH == nil then
		SC.SET.THEALTH = {
			value = false,
			HideCombat = true,
			MidUnder = 'None',
			LowUnder = 'None'
		}
	end
	--[[if SC.SET.EnergySound1 == nil then
		SC.SET.EnergySound1 = 'None'
	end
	if SC.SET.EnergySound2 == nil then
		SC.SET.EnergySound2 = 'None'
	end
	if SC.SET.EnergySound3 == nil then
		SC.SET.EnergySound3 = 'None'
	end]]
	if SC.SET.DPSound == nil then
		SC.SET.DPSound = 'None'
	end
	if SC.SET.Width == nil then
		SC.SET.Width = 287
	end
	if SC.SET.EnergyTrans == nil then
		SC.SET.EnergyTrans = 0
	end
	if SC.SET.BarMargin == nil then
		SC.SET.BarMargin = 2
	end
	if SC.SET.EnergyHeight == nil then
		SC.SET.EnergyHeight = 28
	end
	if SC.SET.IconSize == nil then
		SC.SET.IconSize = 30
	end
	if SC.SET.TimerHeight == nil then
		SC.SET.TimerHeight = 17
	end
	if SC.SET.BarTexture == nil then
		SC.SET.BarTexture = 'Tukui'
	end
	if SC.SET.display.DPTIME == nil then
		SC.SET.display.DPTIME = 10
	end
	if SC.SET.FrameOpacity == nil then
		SC.SET.FrameOpacity = 100
	end
	if SC.SET.soundChannel == nil then
		SC.SET.soundChannel = "Master"
	end
	if SC.SET.Pandemic == nil then
		SC.SET.Pandemic = true
	end
	if SC.SET.Tick == nil then
		SC.SET.Tick = true
	end
	if SC.SET.CP == nil then
		SC.SET.CP = {
			[5] = {
				[1] = {.33, .63, .33},
				[2] = {.46, .63, .35},
				[3] = {.65, .63, .35},
				[4] = {.65, .42, .31},
				[5] = {.69, .31, .31},
			},
			[6] = {
				[1] = {.33, .63, .33},
				[2] = {.46, .63, .35},
				[3] = {.65, .63, .35},
				[4] = {.65, .42, .31},
				[5] = {.69, .31, .31},
				[6] = {.74, .20, .27},
			}
		}
	end
	if SC.SET.energyColor == nil then
		SC.SET.energyColor = {.92, .92, .92}
	end
	if SC.SET.poison.mh == nil or SC.SET.poison.oh == nil or type(SC.SET.poison.mh) ~= "table" or type(SC.SET.poison.oh) ~= "table" then
		SC.SET.poison.mh = {
			[1] = 'DPB',
			[2] = 'IP',
			[3] = 'IP'
		}
		SC.SET.poison.oh = {
			[1] = 'CRP',
			[2] = 'CRP',
			[3] = 'CRP'
		}
	end
	if SC.SET.font == nil then
		SC.SET.font = "Friz Quadrata TT"
	end
	SliceCommander:InitBarArray()
	SliceCommander:InitOtherArray()
	SliceCommander:SetRB()
end


function SliceCommander:InitOtherArray()
	if SC.OTHER == nil then
		SC.OTHER = {}
	end
	if SC.OTHER['AS'] == nil then
		SC.OTHER['AS'] = {
			['obj'] = 0,
			['expire'] = 0,
			['color'] = 0
		}
	end
	if SC.OTHER['POISON'] == nil then
		SC.OTHER['POISON'] = {
			['obj'] = 0,
			['expire'] = 0,
			['color'] = 0,
			['position'] = {
				['xOfs'] = -25,
				['yOfs'] = 150,
				['relativePoint'] = "CENTER",
				['point'] = "CENTER",
				['relativeTo'] = "UIParent"
			}
		}
	end
	if SC.OTHER['SliceCommanderFrame'] == nil then
		SC.OTHER['SliceCommanderFrame'] = {
			['position'] = {
				['xOfs'] = -250,
				['yOfs'] = 75,
				['relativePoint'] = "CENTER",
				['point'] = "CENTER",
				['relativeTo'] = "UIParent"
			}
		}
	end
end

function SliceCommander:InitGlobalBarArray()
	if SC.BARS['HEALTH'] == nil then
		SC.BARS['HEALTH'] = {['color'] = {0, 0, 0}}
	end
	SC.BARS['HEALTH']['target'] = "player"
	if SC.BARS['THEALTH'] == nil then
		SC.BARS['THEALTH'] = {['color'] = {0, 0, 0}}
	end
	SC.BARS['THEALTH']['target'] = "target"
	if SC.BARS['THREAT'] == nil then
		SC.BARS['THREAT'] = {['color'] = {0, 0, 0}}
	end
	SC.BARS['CP'] = {
		['last'] = 0,--last CP has to be init to 0
		['target'] = '',
		['color'] = 0
	}
	
	if SC.BARS['KS'] == nil then--Kidney Shot
		SC.BARS['KS'] = {['color'] = {.89, .73, .52}}
	end
	if SC.BARS['KSCD'] == nil then--Kidney Shot CD
		SC.BARS['KSCD'] = {['color'] = {.89, .73, .52}}
	end
	if SC.BARS['CS'] == nil then--Cheap Shot
		SC.BARS['CS'] = {['color'] = {.89, .73, .2}}
	end
	if SC.BARS['VANCD'] == nil then--Vanish
		SC.BARS['VANCD'] = {['color'] = {.42, .46, .51}}
	end
	if SC.BARS['SS'] == nil then--Shadowstep
		SC.BARS['SS'] = {['color'] = {.76, .28, .78}}
	end
	if SC.BARS['SSCD'] == nil then--Shadowstep CD
		SC.BARS['SSCD'] = {['color'] = {.76, .28, .78}}
	end
	if SC.BARS['FE'] == nil then--Feint
		SC.BARS['FE'] = {['color'] = {.15, .42, .68}}
	end
	if SC.BARS['FECD'] == nil then--Feint CD
		SC.BARS['FECD'] = {['color'] = {.15, .42, .68}}
	end
	if SC.BARS['KICD'] == nil then--Kick
		SC.BARS['KICD'] = {['color'] = {.42, .46, .51}}
	end
	if SC.BARS['MAI'] == nil then--Master Assassin's Initiative
		SC.BARS['MAI'] = {['color'] = {.42, .46, .51}}
	end
	if SC.BARS['MDCD'] == nil then--Marked for Death
		SC.BARS['MDCD'] = {['color'] = {.92, .33, .33}}
	end
	if SC.BARS['DFACD'] == nil then--Death from Above
		SC.BARS['DFACD'] = {['color'] = {.75, .65, .53}}
	end
	if SC.BARS['AC'] == nil then--Alacrity
		SC.BARS['AC'] = {['color'] = {.6, .55, .17}}
	end
	if SC.BARS['CV'] == nil then--Crimson Vial
		SC.BARS['CV'] = {['color'] = {.43, .67, .64}}
	end
	SC.BARS['CV']['tick'] = {
		['obj'] = {},
		['value'] = 1
	}
	if SC.BARS['CVCD'] == nil then--Crimson Vial
		SC.BARS['CVCD'] = {['color'] = {.33, .57, .54}}
	end
	if SC.BARS['CDCD'] == nil then--Cheated Death
		SC.BARS['CDCD'] = {['color'] = {.2, .47, .19}}
	end
	if SC.BARS['SUB'] == nil then--Subterfuge
		SC.BARS['SUB'] = {['color'] = {.04, .81, .82}}
	end
	if SC.BARS['TOTCD'] == nil then--Tricks of the Trade
		SC.BARS['TOTCD'] = {['color'] = {.3, .3, 1}}
	end
	if SC.BARS['COS'] == nil then--Cloak of Shadows
		SC.BARS['COS'] = {['color'] = {.2, .4, .53}}
	end
	if SC.BARS['COSCD'] == nil then--Cloak of Shadows CD
		SC.BARS['COSCD'] = {['color'] = {.12, .32, .45}}
	end
	if SC.BARS['EVA'] == nil then--Evasion
		SC.BARS['EVA'] = {['color'] = {.42, .46, .51}}
	end
	if SC.BARS['EVACD'] == nil then--Evasion CD
		SC.BARS['EVACD'] = {['color'] = {.42, .46, .51}}
	end
	if SC.BARS['SPR'] == nil then--Sprint
		SC.BARS['SPR'] = {['color'] = {.83, .51, .27}}
	end
	if SC.BARS['SPRCD'] == nil then--Sprint CD
		SC.BARS['SPRCD'] = {['color'] = {.83, .51, .27}}
	end
	if SC.BARS['ESSECD'] == nil then--Essence CD
		SC.BARS['ESSECD'] = {['color'] = {.01, .59, .78}}
	end
	
	SliceCommander:InitCovenantBarArray()
end

function SliceCommander:InitCovenantBarArray()
	if SC.BARS['SUMSCD'] == nil then--Summon Steward (Kyrian)
		SC.BARS['SUMSCD'] = {['color'] = {.2, .45, .82}}
	end
	if SC.BARS['FCCD'] == nil then--Fleshcraft (Necrolord)
		SC.BARS['FCCD'] = {['color'] = {.2, .45, .82}}
	end
	if SC.BARS['SOSCD'] == nil then--Soulshape (Night Fae)
		SC.BARS['SOSCD'] = {['color'] = {.2, .45, .82}}
	end
	if SC.BARS['DOSCD'] == nil then--Door of Shadows (Venthyr)
		SC.BARS['DOSCD'] = {['color'] = {.2, .45, .82}}
	end
	
	
	if SC.BARS['ERCD'] == nil then--Echoing Reprimand (Kyrian)
		SC.BARS['ERCD'] = {['color'] = {.2, .45, .82}}
	end
	if SC.BARS['SBSCD'] == nil then--Serrated Bone Spike (Necrolord)
		SC.BARS['SBSCD'] = {['color'] = {.2, .45, .82}}
	end
	if SC.BARS['SBS'] == nil then--Serrated Bone Spike (Necrolord)
		SC.BARS['SBS'] = {['color'] = {.2, .45, .82}}
	end
	if SC.BARS['SP'] == nil then--Sepsis (Night Fae)
		SC.BARS['SP'] = {['color'] = {.2, .45, .82}}
	end
	if SC.BARS['SPCD'] == nil then--Sepsis (Night Fae)
		SC.BARS['SPCD'] = {['color'] = {.2, .45, .82}}
	end
	if SC.BARS['FLA'] == nil then--Flagellation (Venthyr)
		SC.BARS['FLA'] = {['color'] = {.2, .45, .82}}
	end
	if SC.BARS['FLACD'] == nil then--Flagellation (Venthyr)
		SC.BARS['FLACD'] = {['color'] = {.2, .45, .82}}
	end
end

function SliceCommander:InitAssassinationBarArray()
	if SC.BARS['GAR'] == nil then--Garrote
		SC.BARS['GAR'] = {['color'] = {.83, .27, .27}}
	end
	SC.BARS['GAR']['tick'] = {
		['obj'] = {},
		['value'] = 3
	}
	if SC.BARS['GARCD'] == nil then--Garrote CD
		SC.BARS['GARCD'] = {['color'] = {.83, .27, .27}}
	end
	if SC.BARS['RUP'] == nil then--Rupture
		SC.BARS['RUP'] = {['color'] = {.83, .27, .27}}
	end
	SC.BARS['RUP']['tick'] = {
		['obj'] = {},
		['value'] = 2
	}
	if SC.BARS['BA'] == nil then--Blood of the Assassinated
		SC.BARS['BA'] = {['color'] = {.83, .27, .27}}
	end
	if SC.BARS['EXCD'] == nil then--Exsanguinate
		SC.BARS['EXCD'] = {['color'] = {.83, .27, .27}}
	end
	if SC.BARS['DP'] == nil then--Deadly Poison
		SC.BARS['DP'] = {['color'] = {.27, .68, .23}}
	end
	if SC.BARS['ENV'] == nil then--Envenom
		SC.BARS['ENV'] = {['color'] = {.27, .68, .23}}
	end
	if SC.BARS['ST'] == nil then--Surge of Toxins
		SC.BARS['ST'] = {['color'] = {.27, .68, .23}}
	end
	if SC.BARS['KB'] == nil then--Kingsbane
		SC.BARS['KB'] = {['color'] = {.27, .68, .23}}
	end
	if SC.BARS['KBCD'] == nil then--Kingsbane CD
		SC.BARS['KBCD'] = {['color'] = {.27, .68, .23}}
	end
	if SC.BARS['EP'] == nil then--Elaborate Planning
		SC.BARS['EP'] = {['color'] = {.83, .78, .27}}
	end
	if SC.BARS['VEN'] == nil then--Vendetta
		SC.BARS['VEN'] = {['color'] = {.83, .53, .27}}
	end
	if SC.BARS['VENCD'] == nil then--Vendetta CD
		SC.BARS['VENCD'] = {['color'] = {.71, .4, .15}}
	end
	if SC.BARS['BLS'] == nil then--Blindside
		SC.BARS['BLS'] = {['color'] = {.52, .38, .75}}
	end
	if SC.BARS['MA'] == nil then--Master Assassin
		SC.BARS['MA'] = {['color'] = {.42, .46, .51}}
	end
	if SC.BARS['CT'] == nil then--Crimson Tempest
		SC.BARS['CT'] = {['color'] = {.71, .15, .15}}
	end
	SC.BARS['CT']['tick'] = {
		['obj'] = {},
		['value'] = 2
	}
end

function SliceCommander:InitOutlawBarArray()--http://paletton.com/#uid=60s0u0klqqvbNEzh9uFpomItmi0
	if SC.BARS['SND'] == nil then--Slice and Dice
		SC.BARS['SND'] = {['color'] = {.83, .51, .27}}
	end
	if SC.BARS['RB'] == nil then--Roll the Bones
		SC.BARS['RB'] = {['color'] = {.83, .51, .27}}
	end
	if SC.BARS['RBCD'] == nil then--Roll the Bones CD
		SC.BARS['RBCD'] = {['color'] = {.83, .51, .27}}
	end
	if SC.BARS['AR'] == nil then--Adrenaline Rush
		SC.BARS['AR'] = {['color'] = {.83, .64, .27}}
	end
	if SC.BARS['ARCD'] == nil then--Adrenaline Rush CD
		SC.BARS['ARCD'] = {['color'] = {.71, .52, .15}}
	end
	if SC.BARS['CDB'] == nil then--Curse of the Dreadblades
		SC.BARS['CDB'] = {['color'] = {.2, .6, .36}}
	end
	if SC.BARS['CDBCD'] == nil then--Curse of the Dreadblades CD
		SC.BARS['CDBCD'] = {['color'] = {.2, .6, .36}}
	end
	if SC.BARS['GS'] == nil then--Ghostly Strike
		SC.BARS['GS'] = {['color'] = {.81, .27, .3}}
	end
	if SC.BARS['GSCD'] == nil then--Ghostly Strike CD
		SC.BARS['GSCD'] = {['color'] = {.81, .27, .3}}
	end
	if SC.BARS['OP'] == nil then--Opportunity
		SC.BARS['OP'] = {['color'] = {.83, .73, .27}}
	end
	if SC.BARS['GSWW'] == nil then--Greenskin's Waterlogged Wristcuffs
		SC.BARS['GSWW'] = {['color'] = {.81, .27, .3}}
	end
	if SC.BARS['GHCD'] == nil then--Grappling Hook
		SC.BARS['GHCD'] = {['color'] = {.83, .82, .27}}
	end
	if SC.BARS['BE'] == nil then--Between the Eyes
		SC.BARS['BE'] = {['color'] = {.62, .78, .26}}
	end
	if SC.BARS['BECD'] == nil then--Between the Eyes CD
		SC.BARS['BECD'] = {['color'] = {.62, .78, .26}}
	end
	if SC.BARS['KSPCD'] == nil then--Killing Spree
		SC.BARS['KSPCD'] = {['color'] = {.27, .68, .23}}
	end
	if SC.BARS['LD'] == nil then--Loaded Dice
		SC.BARS['LD'] = {['color'] = {.63, .52, .38}}
	end
	if SC.BARS['BF'] == nil then--Blade Flury
		SC.BARS['BF'] = {['color'] = {.66, .22, .47}}
	end
	if SC.BARS['BFCD'] == nil then--Blade Flury CD
		SC.BARS['BFCD'] = {['color'] = {.66, .22, .47}}
	end
	if SC.BARS['BRCD'] == nil then--Blade Rush
		SC.BARS['BRCD'] = {['color'] = {.99, .99, .56}}
	end
	if SC.BARS['GG'] == nil then--Gouge
		SC.BARS['GG'] = {['color'] = {.81, .27, .3}}
	end
	if SC.BARS['GGCD'] == nil then--Gouge CD
		SC.BARS['GGCD'] = {['color'] = {.81, .27, .3}}
	end
end

function SliceCommander:InitSubtletyBarArray()--http://paletton.com/#uid=64r0u0kleqtbzEKgVuIpcmGtdhZ
	if SC.BARS['GB'] == nil then--Goremaw's Bite
		SC.BARS['GB'] = {['color'] = {.52, .18, .54}}
	end
	if SC.BARS['GBCD'] == nil then--Goremaw's Bite CD
		SC.BARS['GBCD'] = {['color'] = {.52, .18, .54}}
	end
	if SC.BARS['SOD'] == nil then--Symbols of Death
		SC.BARS['SOD'] = {['color'] = {.25, .26, .58}}
	end
	if SC.BARS['SODCD'] == nil then--Symbols of Death CD
		SC.BARS['SODCD'] = {['color'] = {.25, .26, .58}}
	end
	if SC.BARS['SB'] == nil then--Shadow Blades
		SC.BARS['SB'] = {['color'] = {.19, .58, .39}}
	end
	if SC.BARS['SBCD'] == nil then--Shadow Blades CD
		SC.BARS['SBCD'] = {['color'] = {.1, .5, .31}}
	end
	if SC.BARS['SD'] == nil then--Shadow Dance
		SC.BARS['SD'] = {['color'] = {.2, .4, .53}}
	end
	if SC.BARS['SDCD'] == nil then--Shadow Dance CD
		SC.BARS['SDCD'] = {['color'] = {.12, .32, .45}}
	end
	if SC.BARS['SHT'] == nil then--Shuriken Tornado
		SC.BARS['SHT'] = {['color'] = {.6, .55, .17}}
	end
	SC.BARS['SHT']['tick'] = {
		['obj'] = {},
		['value'] = 1
	}
	if SC.BARS['SHTCD'] == nil then--Shuriken Tornado CD
		SC.BARS['SHTCD'] = {['color'] = {.6, .55, .17}}
	end
	if SC.BARS['STHCD'] == nil then--Secret Technique
		SC.BARS['STHCD'] = {['color'] = {.52, .18, .54}}
	end
	if SC.BARS['FW'] == nil then--Find Weakness
		SC.BARS['FW'] = {['color'] = {.28, .86, .47}}
	end
end

function SliceCommander:InitBarArray()
	if SC.BARS == nil then
		SC.BARS = {}
	end
	
	SliceCommander:InitGlobalBarArray()
	SliceCommander:InitAssassinationBarArray()
	SliceCommander:InitOutlawBarArray()
	SliceCommander:InitSubtletyBarArray()
	
	for k, v in pairs(SC.BARS) do
		SC.BARS[k]['obj'] = 0
		SC.BARS[k]['pandemic'] = false
		SC.BARS[k]['expire'] = 0
		SC.BARS[k]['name'] = SC_SPELL[k]
		SC.BARS[k]['id'] = SC_SPELLID[k]
	end

	local buff = {'AR', 'SUB', 'EP', 'CV', 'AC', 'FE', 'SND', 'SD', 'SOD', 'SB', 'OP', 'ENV', 'GB', 'GSWW', 'MAI', 'LD', 'BF', 'SHT', 'COS', 'EVA', 'SPR', 'BLS', 'MA', 'SS'}
	for i, v in ipairs(buff) do--BUFF
		SC.BARS[v]['type'] = 1
	end
	
	local debuff = {'GAR', 'RUP', 'DP', 'KS', 'CS', 'VEN', 'GS', 'KB', 'BA', 'ST', 'CT', 'BE', 'GG', 'FW', 'SP', 'FLA', 'SBS'}
	for i, v in ipairs(debuff) do--DEBUFF
		SC.BARS[v]['type'] = 2
	end
	
	local selfdebuff = {'CDCD', 'CDB'}
	for i, v in ipairs(selfdebuff) do--SELF DEBUF
		SC.BARS[v]['type'] = 3
	end
	
	local cd = {'VANCD', 'KSPCD', 'ARCD', 'CDBCD', 'CVCD', 'MDCD', 'GHCD', 'BECD', 'TOTCD', 'SDCD', 'SBCD', 'VENCD', 'GBCD', 'EXCD', 'GARCD', 'KBCD', 'KSCD', 'DFACD', 'KICD', 'SODCD', 'SHTCD', 'STHCD', 'COSCD', 'EVACD', 'SPRCD', 'FECD', 'GSCD', 'BRCD', 'GGCD', 'ESSECD', 'BFCD', 'RBCD', 'SUMSCD', 'FCCD', 'SOSCD', 'DOSCD', 'ERCD', 'SPCD', 'FLACD'}
	for i, v in ipairs(cd) do--CD
		SC.BARS[v]['type'] = 4
	end

	--CHARGE CD
	SC.BARS['SDCD']['type'] = 5
	SC.BARS['SSCD']['type'] = 5
	SC.BARS['SBSCD']['type'] = 5
	
	--MULTIPLE BUFF BAR
	SC.BARS['RB']['type'] = 6
	
	--[[local chargecd = {'SDCD'}
	for i, v in ipairs(chargecd) do
		SC.BARS[v]['type'] = 5
	end
	
	local chargecd = {'RB'}
	for i, v in ipairs(chargecd) do
		SC.BARS[v]['type'] = 6
	end]]
	
	local pandemic = {
		['GAR'] = '18',
		['SND'] = {
			[0] = 0,
			[1] = 12,
			[2] = 18,
			[3] = 24,
			[4] = 30,
			[5] = 36,
			[6] = 42
		},
		['RUP'] = {
			[0] = 0,
			[1] = 8,
			[2] = 12,
			[3] = 16,
			[4] = 20,
			[5] = 24,
			[6] = 28
		},
		['ENV'] = {
			[0] = 0,
			[1] = 2,
			[2] = 3,
			[3] = 4,
			[4] = 5,
			[5] = 6,
			[6] = 7
		},
		['RB'] = {
			[0] = 0,
			[1] = 12,
			[2] = 18,
			[3] = 24,
			[4] = 30,
			[5] = 36,
			[6] = 42
		}
	}
	for i, v in pairs(pandemic) do
		SC.BARS[i]['pandemic'] = true
		SC.BARS[i]['pandemictimer'] = v
		SC.BARS[i]['pandemicobj'] = nil
	end
end